@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,11 +31,14 @@
 #include <cstdint>
 #include <cstdlib>
 #include <cstring>
+#include <fstream>
+#include <iomanip>
 #include <iostream>
 #include <iterator>
 #include <regex>
 #include <stdexcept>
 #include <string>
+#include <system_error>
 #include <thread>
 #include <vector>
 
@@ -53,10 +57,15 @@
 #include "my_inttypes.h"  // ssize_t
 #include "mysql/harness/filesystem.h"
 #include "mysql/harness/net_ts.h"
+#include "mysql/harness/net_ts/impl/socket.h"
+#include "mysql/harness/net_ts/impl/socket_error.h"
 #include "mysql/harness/net_ts/io_context.h"
+#include "mysql/harness/net_ts/socket.h"
+#include "mysql/harness/stdx/filesystem.h"
 #include "mysqlrouter/mysql_session.h"
 #include "mysqlrouter/utils.h"
-#include "temp_dir.h"
+#include "scope_guard.h"
+#include "test/temp_directory.h"
 
 using mysql_harness::Path;
 using namespace std::chrono_literals;
@@ -82,7 +91,7 @@ Path get_cmake_source_dir() {
 
   if (env_value == nullptr) {
     // try a few places
-    result = Path(get_cwd()).join("..");
+    result = Path(stdx::filesystem::current_path().native()).join("..");
     result = Path(result).real_path();
   } else {
     result = Path(env_value).real_path();
@@ -113,98 +122,6 @@ Path get_envvar_path(const std::string &envvar, Path alternative = Path()) {
   return result;
 }
 
-const std::string get_cwd() {
-  char buffer[FILENAME_MAX];
-  if (!getcwd(buffer, FILENAME_MAX)) {
-    throw std::runtime_error("getcwd failed: " + std::string(strerror(errno)));
-  }
-  return std::string(buffer);
-}
-
-const std::string change_cwd(std::string &dir) {
-  auto cwd = get_cwd();
-#ifndef _WIN32
-  if (chdir(dir.c_str()) == -1) {
-#else
-  if (!SetCurrentDirectory(dir.c_str())) {
-#endif
-    throw std::runtime_error("chdir failed: " + mysqlrouter::get_last_error());
-  }
-  return cwd;
-}
-
-size_t read_bytes_with_timeout(int sockfd, void *buffer, size_t n_bytes,
-                               uint64_t timeout_in_ms) {
-  // returns epoch time (aka unix time, etc), expressed in milliseconds
-  auto get_epoch_in_ms = []() -> uint64_t {
-    using namespace std::chrono;
-    time_point<system_clock> now = system_clock::now();
-    return static_cast<uint64_t>(
-        duration_cast<milliseconds>(now.time_since_epoch()).count());
-  };
-
-  // calculate deadline time
-  uint64_t now_in_ms = get_epoch_in_ms();
-  uint64_t deadline_epoch_in_ms = now_in_ms + timeout_in_ms;
-
-  // read until 1 of 3 things happen: enough bytes were read, we time out or
-  // read() fails
-  size_t bytes_read = 0;
-  while (true) {
-#ifndef _WIN32
-    ssize_t res = read(sockfd, static_cast<char *>(buffer) + bytes_read,
-                       n_bytes - bytes_read);
-#else
-    WSASetLastError(0);
-    ssize_t res = recv(sockfd, static_cast<char *>(buffer) + bytes_read,
-                       n_bytes - bytes_read, 0);
-#endif
-
-    if (res == 0) {  // reached EOF?
-      return bytes_read;
-    }
-
-    if (get_epoch_in_ms() > deadline_epoch_in_ms) {
-      throw std::runtime_error("read() timed out");
-    }
-
-    if (res == -1) {
-#ifndef _WIN32
-      if (errno != EAGAIN) {
-        throw std::runtime_error(std::string("read() failed: ") +
-                                 strerror(errno));
-      }
-#else
-      int err_code = WSAGetLastError();
-      if (err_code != 0) {
-        throw std::runtime_error("recv() failed with error: " +
-                                 get_last_error(err_code));
-      }
-
-#endif
-    } else {
-      bytes_read += static_cast<size_t>(res);
-      if (bytes_read >= n_bytes) {
-        assert(bytes_read == n_bytes);
-        return bytes_read;
-      }
-    }
-
-    std::this_thread::sleep_for(std::chrono::milliseconds(10));
-  }
-}
-
-#ifdef _WIN32
-std::string get_last_error(int err_code) {
-  char message[512];
-  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS |
-                    FORMAT_MESSAGE_ALLOCATE_BUFFER,
-                nullptr, err_code, LANG_NEUTRAL, message, sizeof(message),
-                nullptr);
-  return std::string(message);
-}
-#endif
-
 void init_windows_sockets() {
 #ifdef _WIN32
   WSADATA wsaData;
@@ -230,17 +147,13 @@ bool pattern_found(const std::string &s, const std::string &pattern) {
 }
 
 namespace {
-#ifndef _WIN32
-int close_socket(int sock) {
-  ::shutdown(sock, SHUT_RDWR);
-  return close(sock);
+void shut_and_close_socket(net::impl::socket::native_handle_type sock) {
+  const auto shut_both =
+      static_cast<std::underlying_type_t<net::socket_base::shutdown_type>>(
+          net::socket_base::shutdown_type::shutdown_both);
+  net::impl::socket::shutdown(sock, shut_both);
+  net::impl::socket::close(sock);
 }
-#else
-int close_socket(SOCKET sock) {
-  ::shutdown(sock, SD_BOTH);
-  return closesocket(sock);
-}
-#endif
 }  // namespace
 
 bool wait_for_port_ready(uint16_t port, std::chrono::milliseconds timeout,
@@ -273,11 +186,11 @@ bool wait_for_port_ready(uint16_t port, std::chrono::milliseconds timeout,
     auto sock_id =
         socket(ainfo->ai_family, ainfo->ai_socktype, ainfo->ai_protocol);
     if (sock_id < 0) {
-      throw std::runtime_error("wait_for_port_ready(): socket() failed: " +
-                               std::to_string(mysqlrouter::get_socket_errno()));
+      throw std::system_error(net::impl::socket::last_error_code(),
+                              "wait_for_port_ready(): socket() failed");
     }
     std::shared_ptr<void> exit_close_socket(
-        nullptr, [&](void *) { close_socket(sock_id); });
+        nullptr, [&](void *) { shut_and_close_socket(sock_id); });
 
 #ifdef _WIN32
     // On Windows if the port is not ready yet when we try the connect() first
@@ -291,13 +204,11 @@ bool wait_for_port_ready(uint16_t port, std::chrono::milliseconds timeout,
       // if the address is not available, it is a client side problem.
 #ifdef _WIN32
       if (WSAGetLastError() == WSAEADDRNOTAVAIL) {
-        throw std::system_error(mysqlrouter::get_socket_errno(),
-                                std::system_category());
+        throw std::system_error(net::impl::socket::last_error_code());
       }
 #else
       if (errno == EADDRNOTAVAIL) {
-        throw std::system_error(mysqlrouter::get_socket_errno(),
-                                std::generic_category());
+        throw std::system_error(net::impl::socket::last_error_code());
       }
 #endif
       const auto step = std::min(timeout, step_ms);
@@ -309,7 +220,44 @@ bool wait_for_port_ready(uint16_t port, std::chrono::milliseconds timeout,
   return status >= 0;
 }
 
-inline bool is_port_available_fallback(const uint16_t port) {
+bool wait_for_socket_ready(const std::string &socket,
+                           std::chrono::milliseconds timeout) {
+  const auto sock = ::socket(AF_UNIX, SOCK_STREAM, 0);
+  if (sock < 0) {
+    throw std::system_error(net::impl::socket::last_error_code(),
+                            "wait_for_socket_ready(): socket() failed");
+  }
+
+  struct sockaddr_un endpoint_sock;
+
+  endpoint_sock.sun_family = AF_UNIX;
+  strcpy(endpoint_sock.sun_path, socket.c_str());
+  const auto len =
+      strlen(endpoint_sock.sun_path) + sizeof(endpoint_sock.sun_family);
+
+  auto step_ms = 10ms;
+  // Valgrind needs way more time
+  if (getenv("WITH_VALGRIND")) {
+    timeout *= 10;
+    step_ms *= 10;
+  }
+
+  const auto started = std::chrono::steady_clock::now();
+  int status = 0;
+  do {
+    status = connect(sock, (struct sockaddr *)&endpoint_sock, len);
+
+    if (status < 0) {
+      const auto step = std::min(timeout, step_ms);
+      std::this_thread::sleep_for(std::chrono::milliseconds(step));
+      timeout -= step;
+    }
+  } while (status < 0 && timeout > std::chrono::steady_clock::now() - started);
+
+  return status >= 0;
+}
+
+bool is_port_bindable(const uint16_t port) {
   net::io_context io_ctx;
   net::ip::tcp::acceptor acceptor(io_ctx);
 
@@ -334,16 +282,16 @@ inline bool is_port_available_fallback(const uint16_t port) {
   return true;
 }
 
-bool is_port_available(const uint16_t port) {
+bool is_port_unused(const uint16_t port) {
 #if defined(__linux__)
-  const std::string &netstat_cmd{"netstat -tnl"};
+  const std::string netstat_cmd{"netstat -tnl"};
 #elif defined(_WIN32)
-  const std::string &netstat_cmd{"netstat -p tcp -n -a | findstr LISTEN"};
+  const std::string netstat_cmd{"netstat -p tcp -n -a"};
 #elif defined(__sun)
-  const std::string &netstat_cmd{"netstat -n -P tcp | grep LISTEN"};
+  const std::string netstat_cmd{"netstat -na -P tcp"};
 #else
   // BSD and MacOS
-  const std::string &netstat_cmd{"netstat -p tcp -an | grep LISTEN"};
+  const std::string netstat_cmd{"netstat -p tcp -an"};
 #endif
 
   TempDirectory temp_dir;
@@ -352,7 +300,7 @@ bool is_port_available(const uint16_t port) {
   if (std::system(cmd.c_str()) != 0) {
     // netstat command failed, do the check by trying to bind to the port
     // instead
-    return is_port_available_fallback(port);
+    return is_port_bindable(port);
   }
 
   std::ifstream file{filename};
@@ -360,8 +308,26 @@ bool is_port_available(const uint16_t port) {
 
   std::string line;
   while (std::getline(file, line)) {
-    if (pattern_found(line,
-                      "127\\..*[.:]" + std::to_string(port) + "[^\\d]?")) {
+    // Check if netstat output contains listening port <XYZ> given the following
+    // netstat outputs:
+    //
+    // MacOS
+    // tcp46   0   0 *.XYZ             *.*          LISTEN
+    // tcp4    0   0 127.0.0.1.XYZ     *.*          LISTEN
+    //
+    // Windows
+    //  TCP    127.0.0.1:XYZ          0.0.0.0:0              LISTENING
+    //  TCP    0.0.0.0:XYZ            0.0.0.0:0              LISTENING
+    //
+    //  Linux/BSD
+    //  tcp     0    0 0.0.0.0:XYZ       0.0.0.0:*               LISTEN
+    //  tcp     0    0 127.0.0.1:XYZ     0.0.0.0:*               LISTEN
+    //
+    //  SunOS
+    //  *.XYZ                 *.*              0      0  256000      0 LISTEN
+    //  127.0.0.1.XYZ         *.*              0      0  256000      0 LISTEN
+    if (pattern_found(line, "[\\*,0,127]\\..*[.:]" + std::to_string(port) +
+                                "[^\\d].*LISTEN")) {
       return false;
     }
   }
@@ -372,21 +338,22 @@ bool is_port_available(const uint16_t port) {
 static bool wait_for_port(const bool available, const uint16_t port,
                           std::chrono::milliseconds timeout = 2s) {
   const std::chrono::milliseconds step = 50ms;
+  using clock_type = std::chrono::steady_clock;
+  const auto end = clock_type::now() + timeout;
   do {
-    if (available == is_port_available(port)) return true;
+    if (available == is_port_unused(port)) return true;
     std::this_thread::sleep_for(step);
-    timeout -= step;
-  } while (timeout > 0ms);
+  } while (clock_type::now() < end);
   return false;
 }
 
-bool wait_for_port_not_available(const uint16_t port,
-                                 std::chrono::milliseconds timeout) {
+bool wait_for_port_used(const uint16_t port,
+                        std::chrono::milliseconds timeout) {
   return wait_for_port(/*available*/ false, port, timeout);
 }
 
-bool wait_for_port_available(const uint16_t port,
-                             std::chrono::milliseconds timeout) {
+bool wait_for_port_unused(const uint16_t port,
+                          std::chrono::milliseconds timeout) {
   return wait_for_port(/*available*/ true, port, timeout);
 }
 
@@ -408,6 +375,55 @@ void init_keyring(std::map<std::string, std::string> &default_section,
   default_section["master_key_path"] = masterkey_file;
 }
 
+bool wait_file_exists(const std::string &file, const bool exists,
+                      std::chrono::milliseconds timeout) {
+  if (getenv("WITH_VALGRIND")) {
+    timeout *= 10;
+  }
+
+  const auto MSEC_STEP = 20ms;
+  bool found = false;
+  using clock_type = std::chrono::steady_clock;
+  const auto end = clock_type::now() + timeout;
+  do {
+    found = mysql_harness::Path(file).exists();
+    if (found != exists) {
+      auto step = std::min(timeout, MSEC_STEP);
+      std::this_thread::sleep_for(step);
+    }
+  } while (found != exists && clock_type::now() < end);
+
+  return found;
+}
+
+bool is_socket_bindable(const std::string &socket) {
+  net::io_context io_ctx;
+  local::stream_protocol::acceptor sock(io_ctx);
+  sock.set_option(net::socket_base::reuse_address(true));
+
+  const auto open_res = sock.open();
+  if (!open_res) {
+    return false;
+  }
+
+  local::stream_protocol::endpoint ep(socket);
+  const auto bind_res = sock.bind(ep);
+  if (!bind_res) {
+    return false;
+  }
+
+  Scope_guard on_exit([&socket]() { unlink(socket.c_str()); });
+
+  const auto listen_res = sock.listen(128);
+  if (!listen_res) {
+    return false;
+  }
+
+  sock.cancel();
+
+  return true;
+}
+
 namespace {
 
 bool real_find_in_file(
@@ -576,3 +592,71 @@ void connect_client_and_query_port(unsigned router_port, std::string &out_port,
   }
   out_port = std::string((*result)[0]);
 }
+
+// Wait for the nth occurrence of the log_regex in the log_file with the timeout
+// If it's found returns the full line containing the log_regex
+// If the timeout has been reached returns unexpected
+static std::optional<std::string> wait_log_line(
+    const std::string &log_file, const std::string &log_regex,
+    const unsigned n_occurence = 1,
+    const std::chrono::milliseconds timeout = 1s) {
+  const auto start_timestamp = std::chrono::steady_clock::now();
+  const auto kStep = 50ms;
+
+  do {
+    std::istringstream ss{get_file_output(log_file)};
+
+    unsigned current_occurence = 0;
+    for (std::string line; std::getline(ss, line);) {
+      if (pattern_found(line, log_regex)) {
+        current_occurence++;
+        if (current_occurence == n_occurence) return {line};
+      }
+    }
+
+    if (std::chrono::duration_cast<std::chrono::milliseconds>(
+            std::chrono::steady_clock::now() - start_timestamp) >= timeout) {
+      return std::nullopt;
+    }
+    std::this_thread::sleep_for(kStep);
+  } while (true);
+}
+
+std::optional<std::chrono::time_point<std::chrono::system_clock>>
+get_log_timestamp(const std::string &log_file, const std::string &log_regex,
+                  const unsigned occurence,
+                  const std::chrono::milliseconds timeout) {
+  // first wait for the nth occurrence of the pattern
+  const auto log_line = wait_log_line(log_file, log_regex, occurence, timeout);
+  if (!log_line) {
+    return std::nullopt;
+  }
+
+  const std::string log_line_str = log_line.value();
+  // make sure the line is prefixed with the expected timestamp
+  // 2020-06-09 03:53:26.027 foo bar
+  if (!pattern_found(log_line_str,
+                     "^\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}\\.\\d{3}.*")) {
+    return std::nullopt;
+  }
+
+  // extract the timestamp prefix and convert to the duration
+  std::string timestamp_str =
+      log_line_str.substr(0, strlen("2020-06-09 03:53:26.027"));
+  std::tm tm{};
+#ifdef HAVE_STRPTIME
+  char *rest = strptime(timestamp_str.c_str(), "%Y-%m-%d %H:%M:%S", &tm);
+  assert(*rest == '.');
+  int milliseconds = atoi(++rest);
+#else
+  std::stringstream timestamp_ss(timestamp_str);
+  char dot;
+  unsigned milliseconds;
+  timestamp_ss >> std::get_time(&tm, "%Y-%m-%d %H:%M:%S") >> dot >>
+      milliseconds;
+#endif
+  auto result = std::chrono::system_clock::from_time_t(std::mktime(&tm));
+  result += std::chrono::milliseconds(milliseconds);
+
+  return result;
+}
