@@ -24,10 +24,14 @@ want_syslog=0
 syslog_tag=
 user='@MYSQLD_USER@'
 pid_file=
+pid_file_append=
 err_log=
+err_log_append=
+timestamp_format=UTC
 
 syslog_tag_mysqld=mysqld
 syslog_tag_mysqld_safe=mysqld_safe
+syslog_facility=daemon
 
 trap '' 1 2 3 15			# we shouldn't let anyone kill us
 trap '' 13                              # not even SIGPIPE
@@ -85,6 +89,10 @@ Usage: $0 [OPTIONS]
   --syslog                   Log messages to syslog with 'logger'
   --skip-syslog              Log messages to error log (default)
   --syslog-tag=TAG           Pass -t "mysqld-TAG" to 'logger'
+  --mysqld-safe-log-         TYPE must be one of UTC (ISO 8601 UTC),
+    timestamps=TYPE          system (ISO 8601 local time), hyphen
+                             (hyphenated date a la mysqld 5.6), legacy
+                             (legacy non-ISO 8601 mysqld_safe timestamps)
 
 All other options are passed to the mysqld program.
 
@@ -125,7 +133,7 @@ log_generic () {
   priority="$1"
   shift
 
-  msg="`date +'%y%m%d %H:%M:%S'` mysqld_safe $*"
+  msg="`eval $DATE` mysqld_safe $*"
   echo "$msg"
   case $logging in
     init) ;;  # Just echo the message, don't save it anywhere
@@ -137,6 +145,14 @@ log_generic () {
       fi
       ;;
     syslog) logger -t "$syslog_tag_mysqld_safe" -p "$priority" "$*" ;;
+    both)
+      if [ -w / -o "$USER" = "root" ]; then
+        true
+      else
+        echo "$msg" >> "$err_log"
+      fi
+      logger -t "$syslog_tag_mysqld_safe" -p "$priority" "$*"
+      ;;
     *)
       echo "Internal program error (non-fatal):" \
            " unknown logging method '$logging'" >&2
@@ -145,11 +161,11 @@ log_generic () {
 }
 
 log_error () {
-  log_generic daemon.error "$@" >&2
+  log_generic ${syslog_facility}.error "$@" >&2
 }
 
 log_notice () {
-  log_generic daemon.notice "$@"
+  log_generic ${syslog_facility}.notice "$@"
 }
 
 eval_log_error () {
@@ -163,13 +179,14 @@ eval_log_error () {
       fi
       ;;
     syslog)
-      # mysqld often prefixes its messages with a timestamp, which is
-      # redundant when logging to syslog (which adds its own timestamp)
-      # However, we don't strip the timestamp with sed here, because
-      # sed buffers output (only GNU sed supports a -u (unbuffered) option)
-      # which means that messages may not get sent to syslog until the
-      # mysqld process quits.
-      cmd="$cmd 2>&1 | logger -t '$syslog_tag_mysqld' -p daemon.error"
+      cmd="$cmd --log-syslog=1 --log-syslog-facility=$syslog_facility '--log-syslog-tag=$syslog_tag' > /dev/null 2>&1"
+      ;;
+    both)
+      if [ -w / -o "$USER" = "root" ]; then
+        cmd="$cmd --log-syslog=1 --log-syslog-facility=$syslog_facility '--log-syslog-tag=$syslog_tag' > /dev/null 2>&1"
+      else
+        cmd="$cmd --log-syslog=1 --log-syslog-facility=$syslog_facility '--log-syslog-tag=$syslog_tag' >> "`shell_quote_string "$err_log"`" 2>&1"
+      fi
       ;;
     *)
       echo "Internal program error (non-fatal):" \
@@ -178,6 +195,7 @@ eval_log_error () {
   esac
 
   #echo "Running mysqld: [$cmd]"
+  cmd="env MYSQLD_PARENT_PID=$$ $cmd"
   eval "$cmd"
 }
 
@@ -258,6 +276,7 @@ parse_arguments() {
       --open-files-limit=*) open_files="$val" ;;
       --open_files_limit=*) open_files="$val" ;;
       --skip-kill-mysqld*) KILL_MYSQLD=0 ;;
+      --mysqld-safe-log-timestamps=*) timestamp_format="$val" ;;
       --syslog) want_syslog=1 ;;
       --skip-syslog) want_syslog=0 ;;
       --syslog-tag=*) syslog_tag="$val" ;;
@@ -335,8 +354,6 @@ mysqld_ld_preload_text() {
 
 # set_malloc_lib LIB
 # - If LIB is empty, do nothing and return
-# - If LIB is 'tcmalloc', look for tcmalloc shared library in $malloc_dirs.
-#   tcmalloc is part of the Google perftools project.
 # - If LIB is an absolute path, assume it is a malloc shared library
 #
 # Put LIB in mysqld_ld_preload, which will be added to LD_PRELOAD when
@@ -346,24 +363,6 @@ set_malloc_lib() {
   malloc_dirs="/usr/lib /usr/lib64 /usr/lib/i386-linux-gnu /usr/lib/x86_64-linux-gnu"
   malloc_lib="$1"
 
-  if [ "$malloc_lib" = tcmalloc ]; then
-    malloc_lib=
-    for libdir in `echo $malloc_dirs`; do
-      for flavor in _minimal '' _and_profiler _debug; do
-        tmp="$libdir/libtcmalloc$flavor.so"
-        #log_notice "DEBUG: Checking for malloc lib '$tmp'"
-        [ -r "$tmp" ] || continue
-        malloc_lib="$tmp"
-        break 2
-      done
-    done
-
-    if [ -z "$malloc_lib" ]; then
-      log_error "no shared library for --malloc-lib=tcmalloc found in $malloc_dirs"
-      exit 1
-    fi
-  fi
-
   # Allow --malloc-lib='' to override other settings
   [ -z  "$malloc_lib" ] && return
 
@@ -387,8 +386,7 @@ set_malloc_lib() {
       esac
       ;;
     *)
-      log_error "--malloc-lib must be an absolute path or 'tcmalloc'; " \
-        "ignoring value '$malloc_lib'"
+      log_error "--malloc-lib must be an absolute path ignoring value '$malloc_lib'"
       exit 1
       ;;
   esac
@@ -468,10 +466,6 @@ fi
 if test -d $MY_BASEDIR_VERSION/data/mysql
 then
   DATADIR=$MY_BASEDIR_VERSION/data
-  if test -z "$defaults" -a -r "$DATADIR/my.cnf"
-  then
-    defaults="--defaults-extra-file=$DATADIR/my.cnf"
-  fi
 # Or just give up and use our compiled-in default
 else
   DATADIR=@localstatedir@
@@ -479,23 +473,7 @@ fi
 
 if test -z "$MYSQL_HOME"
 then 
-  if test -r "$MY_BASEDIR_VERSION/my.cnf" && test -r "$DATADIR/my.cnf"
-  then
-    log_error "WARNING: Found two instances of my.cnf -
-$MY_BASEDIR_VERSION/my.cnf and
-$DATADIR/my.cnf
-IGNORING $DATADIR/my.cnf"
-
-    MYSQL_HOME=$MY_BASEDIR_VERSION
-  elif test -r "$DATADIR/my.cnf"
-  then
-    log_error "WARNING: Found $DATADIR/my.cnf
-The data directory is a deprecated location for my.cnf, please move it to
-$MY_BASEDIR_VERSION/my.cnf"
-    MYSQL_HOME=$DATADIR
-  else
-    MYSQL_HOME=$MY_BASEDIR_VERSION
-  fi
+  MYSQL_HOME=$MY_BASEDIR_VERSION
 fi
 export MYSQL_HOME
 
@@ -528,6 +506,19 @@ fi
 parse_arguments `$print_defaults $defaults --loose-verbose mysqld_safe safe_mysqld`
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 
+#
+# Sort out date command from $timestamp_format early so we'll start off
+# with correct log messages.
+#
+case "$timestamp_format" in
+    UTC|utc)       DATE="date -u +%Y-%m-%dT%H:%M:%S.%06NZ";;
+    SYSTEM|system) DATE="date +%Y-%m-%dT%H:%M:%S.%06N%:z";;
+    HYPHEN|hyphen) DATE="date +'%Y-%m-%d %H:%M:%S'";;
+    LEGACY|legacy) DATE="date +'%y%m%d %H:%M:%S'";;
+    *)             DATE="date -u +%Y-%m-%dT%H:%M:%S.%06NZ";
+                   log_error "unknown data format $timestamp_format, using UTC";;
+esac
+
 #
 # Try to find the plugin directory
 #
@@ -564,6 +555,19 @@ then
   fi
 fi
 
+if [ $want_syslog -eq 1 ]
+then
+  if [ -n "$syslog_tag" ]
+  then
+    # Sanitize the syslog tag
+    syslog_tag=`echo "$syslog_tag" | sed -e 's/[^a-zA-Z0-9_-]/_/g'`
+    syslog_tag_mysqld_safe="${syslog_tag_mysqld_safe}-$syslog_tag"
+    syslog_tag_mysqld="${syslog_tag_mysqld}-$syslog_tag"
+  fi
+  log_notice "Logging to syslog."
+  logging=syslog
+fi
+
 if [ -n "$err_log" -o $want_syslog -eq 0 ]
 then
   if [ -n "$err_log" ]
@@ -582,34 +586,50 @@ then
       err_log="$err_log".err
     fi
 
+    err_log_append="$err_log"
     case "$err_log" in
       /* ) ;;
+      ./*|../*)
+          # Preparing absolute path from the relative path value specified for the
+          # --log-error argument.
+          #
+          # Absolute path will be prepared for the value of following form
+          #    ./bar/foo OR
+          #   ../old_bar/foo
+          # for --log-error argument.
+          #
+          # Note: If directory of log file name does not exists or
+          #       if write or execute permissions are missing on directory then
+          #       --log-error is set  $DATADIR/`hostname`.err
+
+          log_dir_name="$(dirname "$err_log")";
+          if [ ! -d "$log_dir_name" ]
+          then
+            log_notice "Directory "$log_dir_name" does not exists.";
+            err_log=$DATADIR/`hostname`.err
+          elif [ ! -x "$log_dir_name" -o ! -w "$log_dir_name" ]
+          then
+            log_notice "Do not have Execute or Write permissions on directory "$log_dir_name".";
+            err_log=$DATADIR/`hostname`.err
+          else
+            err_log=$(cd $log_dir_name && pwd -P)/$(basename "$err_log")
+          fi
+          ;;
       * ) err_log="$DATADIR/$err_log" ;;
     esac
   else
-    err_log=$DATADIR/`@HOSTNAME@`.err
+    err_log=$DATADIR/`hostname`.err
+    err_log_append=`hostname`.err
   fi
 
-  append_arg_to_args "--log-error=$err_log"
+  append_arg_to_args "--log-error=$err_log_append"
 
   if [ $want_syslog -eq 1 ]
   then
-    # User explicitly asked for syslog, so warn that it isn't used
-    log_error "Can't log to error log and syslog at the same time.  Remove all --log-error configuration options for --syslog to take effect."
+    logging=both
+  else
+    logging=file
   fi
-
-  # Log to err_log file
-  logging=file
-else
-  if [ -n "$syslog_tag" ]
-  then
-    # Sanitize the syslog tag
-    syslog_tag=`echo "$syslog_tag" | sed -e 's/[^a-zA-Z0-9_-]/_/g'`
-    syslog_tag_mysqld_safe="${syslog_tag_mysqld_safe}-$syslog_tag"
-    syslog_tag_mysqld="${syslog_tag_mysqld}-$syslog_tag"
-  fi
-  log_notice "Logging to syslog."
-  logging=syslog
 fi
 
 logdir=`dirname "$err_log"`
@@ -699,14 +719,16 @@ fi
 
 if test -z "$pid_file"
 then
-  pid_file="$DATADIR/`@HOSTNAME@`.pid"
+  pid_file="$DATADIR/`hostname`.pid"
+  pid_file_append="`hostname`.pid"
 else
+  pid_file_append="$pid_file"
   case "$pid_file" in
     /* ) ;;
     * )  pid_file="$DATADIR/$pid_file" ;;
   esac
 fi
-append_arg_to_args "--pid-file=$pid_file"
+append_arg_to_args "--pid-file=$pid_file_append"
 
 if test -n "$mysql_unix_port"
 then
@@ -806,6 +828,16 @@ mysqld daemon not started"
         log_error "Fatal error: Can't remove the socket file:
 $safe_mysql_unix_port.
 Please remove the file manually and start $0 again;
+mysqld daemon not started"
+        exit 1
+      fi
+  fi
+  if [ ! -h "$pid_file.shutdown" ]; then
+      rm -f "$pid_file.shutdown"
+      if test -f "$pid_file.shutdown"; then
+        log_error "Fatal error: Can't remove the shutdown file:
+$pid_file.shutdown.
+Please remove the file manually and start $0 again;
 mysqld daemon not started"
         exit 1
       fi
@@ -845,16 +877,20 @@ log_notice "Starting $MYSQLD daemon with databases from $DATADIR"
 
 # variable to track the current number of "fast" (a.k.a. subsecond) restarts
 fast_restart=0
-# maximum number of restarts before trottling kicks in
+# maximum number of restarts before throttling kicks in
 max_fast_restarts=5
 # flag whether a usable sleep command exists
 have_sleep=1
-
 while true
 do
   start_time=`date +%M%S`
-
   eval_log_error "$cmd"
+  if [ $? -eq 16 ] ; then
+    dont_restart_mysqld=false
+    echo "Restarting mysqld..."
+  else
+    dont_restart_mysqld=true
+  fi
 
   # hypothetical: log was renamed but not
   # flushed yet. we'd recreate it with
@@ -884,18 +920,26 @@ do
 
   end_time=`date +%M%S`
 
-  if test ! -f "$pid_file"		# This is removed if normal shutdown
-  then
-    break
-  else                                  # self's mysqld crashed or other's mysqld running
-    PID=`cat "$pid_file"`
-    if @CHECK_PID@
-    then                                # true when above pid belongs to a running mysqld process
-      log_error "A mysqld process with pid=$PID is already running. Aborting!!"
-      exit 1
+  if $dont_restart_mysqld; then
+    if test ! -f "$pid_file"		# This is removed if normal shutdown
+    then
+      break
+    else                                  # self's mysqld crashed or other's mysqld running
+        PID=`cat "$pid_file"`
+        if @CHECK_PID@
+        then                                # true when above pid belongs to a running mysqld process
+          log_error "A mysqld process with pid=$PID is already running. Aborting!!"
+          exit 1
+        fi
     fi
   fi
 
+  if test -f "$pid_file.shutdown"	# created to signal that it must stop
+  then
+    log_notice "$pid_file.shutdown present. The server will not restart."
+    break
+  fi
+
 
   # sanity check if time reading is sane and there's sleep
   if test $end_time -gt 0 -a $have_sleep -gt 0
@@ -911,7 +955,7 @@ do
         sleep_state=$?
         if test $sleep_state -gt 0
         then
-          log_notice "The server is respawning too fast and no working sleep command. Turning off trottling."
+          log_notice "The server is respawning too fast and in addition no working 'sleep' command was found. Turning off throttling."
           have_sleep=0
         fi
 
@@ -955,10 +999,20 @@ do
     rm -f "$pid_file"
   fi
   if [ ! -h "$safe_mysql_unix_port" ]; then
-   rm -f "$safe_mysql_unix_port"
+    rm -f "$safe_mysql_unix_port"
+  fi
+  if [ ! -h "$pid_file.shutdown" ]; then
+    rm -f "$pid_file.shutdown"
   fi
   log_notice "mysqld restarted"
 done
 
+if [ ! -h "$pid_file.shutdown" ]; then
+  rm -f "$pid_file.shutdown"
+fi
+
 log_notice "mysqld from pid file $pid_file ended"
 
+if [ ! -h "$safe_pid" ]; then
+  rm -f "$safe_pid"                       # Some Extra Safety. File is deleted
+fi                                        # once the mysqld process ends.
