@@ -1,16 +1,17 @@
 #ifndef BINLOG_H_INCLUDED
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,15 +37,16 @@
 #include "my_io.h"
 #include "my_sharedlib.h"
 #include "my_sys.h"
-#include "mysql/components/services/mysql_cond_bits.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/psi_cond_bits.h"
-#include "mysql/components/services/psi_file_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
+#include "mysql/components/services/bits/mysql_cond_bits.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/psi_cond_bits.h"
+#include "mysql/components/services/bits/psi_file_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/udf_registration_types.h"
-#include "mysql_com.h"  // Item_result
+#include "mysql_com.h"          // Item_result
+#include "sql/binlog_reader.h"  // Binlog_file_reader
 #include "sql/rpl_commit_stage_manager.h"
 #include "sql/rpl_trx_tracking.h"
 #include "sql/tc_log.h"            // TC_LOG
@@ -102,17 +104,14 @@ struct Binlog_user_var_event {
 #define LOG_INFO_FATAL -7
 #define LOG_INFO_IN_USE -8
 #define LOG_INFO_EMFILE -9
+#define LOG_INFO_BACKUP_LOCK -10
+#define LOG_INFO_NOT_IN_USE -11
 
 /* bitmap to MYSQL_BIN_LOG::close() */
 #define LOG_CLOSE_INDEX 1
 #define LOG_CLOSE_TO_BE_OPENED 2
 #define LOG_CLOSE_STOP_EVENT 4
 
-/*
-  Note that we destroy the lock mutex in the destructor here.
-  This means that object instances cannot be destroyed/go out of scope
-  until we have reset thd->current_linfo to NULL;
- */
 struct LOG_INFO {
   char log_file_name[FN_REFLEN] = {0};
   my_off_t index_file_offset, index_file_start_offset;
@@ -120,13 +119,15 @@ struct LOG_INFO {
   bool fatal;       // if the purge happens to give us a negative offset
   int entry_index;  // used in purge_logs(), calculatd in find_log_pos().
   int encrypted_header_size;
+  my_thread_id thread_id;
   LOG_INFO()
       : index_file_offset(0),
         index_file_start_offset(0),
         pos(0),
         fatal(false),
         entry_index(0),
-        encrypted_header_size(0) {
+        encrypted_header_size(0),
+        thread_id(0) {
     memset(log_file_name, 0, FN_REFLEN);
   }
 };
@@ -160,26 +161,36 @@ class MYSQL_BIN_LOG : public TC_LOG {
   PSI_mutex_key m_key_LOCK_binlog_end_pos;
   /** The PFS instrumentation key for @ LOCK_commit_queue. */
   PSI_mutex_key m_key_LOCK_commit_queue;
+  /** The PFS instrumentation key for @ LOCK_after_commit_queue. */
+  PSI_mutex_key m_key_LOCK_after_commit_queue;
   /** The PFS instrumentation key for @ LOCK_done. */
   PSI_mutex_key m_key_LOCK_done;
   /** The PFS instrumentation key for @ LOCK_flush_queue. */
   PSI_mutex_key m_key_LOCK_flush_queue;
   /** The PFS instrumentation key for @ LOCK_sync_queue. */
   PSI_mutex_key m_key_LOCK_sync_queue;
+  /** The PFS instrumentation key for @ LOCK_wait_for_group_turn. */
+  PSI_mutex_key m_key_LOCK_wait_for_group_turn;
   /** The PFS instrumentation key for @ COND_done. */
   PSI_mutex_key m_key_COND_done;
   /** The PFS instrumentation key for @ COND_flush_queue. */
   PSI_mutex_key m_key_COND_flush_queue;
   /** The instrumentation key to use for @ LOCK_commit. */
   PSI_mutex_key m_key_LOCK_commit;
+  /** The instrumentation key to use for @ LOCK_after_commit. */
+  PSI_mutex_key m_key_LOCK_after_commit;
   /** The instrumentation key to use for @ LOCK_sync. */
   PSI_mutex_key m_key_LOCK_sync;
   /** The instrumentation key to use for @ LOCK_xids. */
   PSI_mutex_key m_key_LOCK_xids;
+  /** The instrumentation key to use for @ m_key_LOCK_log_info. */
+  PSI_mutex_key m_key_LOCK_log_info;
   /** The instrumentation key to use for @ update_cond. */
   PSI_cond_key m_key_update_cond;
   /** The instrumentation key to use for @ prep_xids_cond. */
   PSI_cond_key m_key_prep_xids_cond;
+  /** The PFS instrumentation key for @ COND_wait_for_group_turn. */
+  PSI_cond_key m_key_COND_wait_for_group_turn;
   /** The instrumentation key to use for opening the log file. */
   PSI_file_key m_key_file_log;
   /** The instrumentation key to use for opening the log index file. */
@@ -192,6 +203,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
   /* POSIX thread objects are inited by init_pthread_objects() */
   mysql_mutex_t LOCK_index;
   mysql_mutex_t LOCK_commit;
+  mysql_mutex_t LOCK_after_commit;
   mysql_mutex_t LOCK_sync;
   mysql_mutex_t LOCK_binlog_end_pos;
   mysql_mutex_t LOCK_xids;
@@ -265,6 +277,17 @@ class MYSQL_BIN_LOG : public TC_LOG {
   int new_file_without_locking(
       Format_description_log_event *extra_description_event);
 
+  /**
+    Checks whether binlog caches are disabled (binlog does not cache data) or
+    empty in case binloggging is enabled in the current call to this function.
+    This function may be safely called in case binlogging is disabled.
+    @retval true binlog local caches are empty or disabled and binlogging is
+    enabled
+    @retval false binlog local caches are enabled and contain data or binlogging
+    is disabled
+  */
+  bool is_current_stmt_binlog_enabled_and_caches_empty(const THD *thd) const;
+
  private:
   int new_file_impl(bool need_lock,
                     Format_description_log_event *extra_description_event);
@@ -275,11 +298,26 @@ class MYSQL_BIN_LOG : public TC_LOG {
                                   uint32 new_index_number);
   int generate_new_name(char *new_name, const char *log_name,
                         uint32 new_index_number = 0);
+  /**
+   * Read binary log stream header and Format_desc event from
+   * binlog_file_reader. Check for LOG_EVENT_BINLOG_IN_USE_F flag.
+   * @param[in] binlog_file_reader
+   * @return true - LOG_EVENT_BINLOG_IN_USE_F is set
+   *         false - LOG_EVENT_BINLOG_IN_USE_F is not set or an error occurred
+   *                 while reading log events
+   */
+  bool read_binlog_in_use_flag(Binlog_file_reader &binlog_file_reader);
+
+ protected:
+  /**
+  @brief Notifies waiting threads that binary log has been updated
+  */
+  void signal_update();
 
  public:
   const char *generate_name(const char *log_name, const char *suffix,
                             char *buff);
-  bool is_open() { return atomic_log_state != LOG_CLOSED; }
+  bool is_open() const { return atomic_log_state != LOG_CLOSED; }
 
   /* This is relay log */
   bool is_relay_log;
@@ -300,12 +338,12 @@ class MYSQL_BIN_LOG : public TC_LOG {
 
     FD     - Format-Description event,
     R      - Rotate event
-    R_f    - the fake Rotate event
-    E      - an arbirary event
+    R_f    - The fake Rotate event
+    E      - An arbitrary event
 
     The underscore indexes for any event
-    `_s'   indicates the event is generated by Slave
-    `_m'   - by Master
+    `_s'   - Indicates the event is generated by the Replica
+    `_m'   - By the Source
 
     Two special underscore indexes of FD:
     FD_q   - Format Description event for queuing   (relay-logging)
@@ -325,18 +363,23 @@ class MYSQL_BIN_LOG : public TC_LOG {
 
   void set_psi_keys(
       PSI_mutex_key key_LOCK_index, PSI_mutex_key key_LOCK_commit,
-      PSI_mutex_key key_LOCK_commit_queue, PSI_mutex_key key_LOCK_done,
+      PSI_mutex_key key_LOCK_commit_queue, PSI_mutex_key key_LOCK_after_commit,
+      PSI_mutex_key key_LOCK_after_commit_queue, PSI_mutex_key key_LOCK_done,
       PSI_mutex_key key_LOCK_flush_queue, PSI_mutex_key key_LOCK_log,
       PSI_mutex_key key_LOCK_binlog_end_pos, PSI_mutex_key key_LOCK_sync,
       PSI_mutex_key key_LOCK_sync_queue, PSI_mutex_key key_LOCK_xids,
-      PSI_cond_key key_COND_done, PSI_cond_key key_COND_flush_queue,
-      PSI_cond_key key_update_cond, PSI_cond_key key_prep_xids_cond,
-      PSI_file_key key_file_log, PSI_file_key key_file_log_index,
-      PSI_file_key key_file_log_cache, PSI_file_key key_file_log_index_cache) {
+      PSI_mutex_key key_LOCK_log_info,
+      PSI_mutex_key key_LOCK_wait_for_group_turn, PSI_cond_key key_COND_done,
+      PSI_cond_key key_COND_flush_queue, PSI_cond_key key_update_cond,
+      PSI_cond_key key_prep_xids_cond,
+      PSI_cond_key key_COND_wait_for_group_turn, PSI_file_key key_file_log,
+      PSI_file_key key_file_log_index, PSI_file_key key_file_log_cache,
+      PSI_file_key key_file_log_index_cache) {
     m_key_COND_done = key_COND_done;
     m_key_COND_flush_queue = key_COND_flush_queue;
 
     m_key_LOCK_commit_queue = key_LOCK_commit_queue;
+    m_key_LOCK_after_commit_queue = key_LOCK_after_commit_queue;
     m_key_LOCK_done = key_LOCK_done;
     m_key_LOCK_flush_queue = key_LOCK_flush_queue;
     m_key_LOCK_sync_queue = key_LOCK_sync_queue;
@@ -345,20 +388,35 @@ class MYSQL_BIN_LOG : public TC_LOG {
     m_key_LOCK_log = key_LOCK_log;
     m_key_LOCK_binlog_end_pos = key_LOCK_binlog_end_pos;
     m_key_LOCK_commit = key_LOCK_commit;
+    m_key_LOCK_after_commit = key_LOCK_after_commit;
     m_key_LOCK_sync = key_LOCK_sync;
     m_key_LOCK_xids = key_LOCK_xids;
+    m_key_LOCK_log_info = key_LOCK_log_info;
     m_key_update_cond = key_update_cond;
     m_key_prep_xids_cond = key_prep_xids_cond;
     m_key_file_log = key_file_log;
     m_key_file_log_index = key_file_log_index;
     m_key_file_log_cache = key_file_log_cache;
     m_key_file_log_index_cache = key_file_log_index_cache;
+
+    m_key_LOCK_wait_for_group_turn = key_LOCK_wait_for_group_turn;
+    m_key_COND_wait_for_group_turn = key_COND_wait_for_group_turn;
   }
 
  public:
   /** Manage the MTS dependency tracking */
   Transaction_dependency_tracker m_dependency_tracker;
 
+  /**
+    Find the oldest binary log referenced by the index file
+
+    @param[out] binlog_file_name the file name of oldest log found
+    @param[out] errmsg the error message outputted, which is left untouched
+                if the function returns false
+    @return false on success, true on error.
+  */
+  bool find_first_log(std::string &binlog_file_name, std::string &errmsg);
+
   /**
     Find the oldest binary log that contains any GTID that
     is not in the given gtid set.
@@ -373,7 +431,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
   */
   bool find_first_log_not_in_gtid_set(char *binlog_file_name,
                                       const Gtid_set *gtid_set,
-                                      Gtid *first_gtid, const char **errmsg);
+                                      Gtid *first_gtid, std::string &errmsg);
 
   /**
     Reads the set of all GTIDs in the binary/relay log, and the set
@@ -565,7 +623,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     Pre-condition: transactions should have called ha_prepare_low, using
                    HA_IGNORE_DURABILITY, before entering here.
 
-    Stage#0 implements slave-preserve-commit-order for applier threads that
+    Stage#0 implements replica-preserve-commit-order for applier threads that
     write the binary log. i.e. it forces threads to enter the queue in the
     correct commit order.
 
@@ -588,7 +646,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     will wait until the last stage is finished.
 
     Stage 0 (SLAVE COMMIT ORDER):
-    1. If slave-preserve-commit-order and is slave applier worker thread, then
+    1. If replica-preserve-commit-order and is slave applier worker thread, then
        waits until its turn to commit i.e. till it is on the top of the queue.
     2. When it reaches top of the queue, it signals next worker in the commit
        order queue to awake.
@@ -653,16 +711,34 @@ class MYSQL_BIN_LOG : public TC_LOG {
   void reset_bytes_written() { bytes_written = 0; }
   void harvest_bytes_written(Relay_log_info *rli, bool need_log_space_lock);
   void set_max_size(ulong max_size_arg);
-  void signal_update() {
-    DBUG_TRACE;
-    mysql_cond_broadcast(&update_cond);
-    return;
-  }
 
   void update_binlog_end_pos(bool need_lock = true);
   void update_binlog_end_pos(const char *file, my_off_t pos);
 
-  int wait_for_update(const struct timespec *timeout);
+  /**
+    Wait until we get a signal that the binary log has been updated.
+
+    NOTES
+    @param[in] timeout    a pointer to a timespec;
+                          NULL means to wait w/o timeout.
+    @retval    0          if got signalled on update
+    @retval    non-0      if wait timeout elapsed
+    @note
+      LOCK_binlog_end_pos must be owned before calling this function, may be
+      temporarily released while the thread is waiting and is reacquired before
+      returning from the function
+  */
+  int wait_for_update(const std::chrono::nanoseconds &timeout);
+
+  /**
+    Wait until we get a signal that the binary log has been updated.
+    @retval    0          success
+    @note
+      LOCK_binlog_end_pos must be owned before calling this function, may be
+      temporarily released while the thread is waiting and is reacquired before
+      returning from the function
+  */
+  int wait_for_update();
 
  public:
   void init_pthread_objects();
@@ -717,7 +793,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
      Gtid_log_event and BEGIN, COMMIT automatically.
 
      It is aimed to handle cases of "background" logging where a statement is
-     logged indirectly, like "DELETE FROM a_memory_table". So don't use it on
+     logged indirectly, like "TRUNCATE TABLE a_memory_table". So don't use it on
      any normal statement.
 
      @param[in] thd  the THD object of current thread.
@@ -727,8 +803,8 @@ class MYSQL_BIN_LOG : public TC_LOG {
 
      @return Returns false if succeeds, otherwise true is returned.
   */
-  bool write_dml_directly(THD *thd, const char *stmt, size_t stmt_len,
-                          enum enum_sql_command sql_command);
+  bool write_stmt_directly(THD *thd, const char *stmt, size_t stmt_len,
+                           enum enum_sql_command sql_command);
 
   void report_cache_write_error(THD *thd, bool is_transactional);
   bool check_write_error(const THD *thd);
@@ -746,15 +822,59 @@ class MYSQL_BIN_LOG : public TC_LOG {
   bool write_buffer(const char *buf, uint len, Master_info *mi);
   bool write_event(Log_event *ev, Master_info *mi);
 
+  /**
+     Logging XA commit/rollback of a prepared transaction.
+
+     It fills in the appropriate event in the statement cache whenever xid
+     state is marked with is_binlogged() flag that indicates the prepared
+     part of the transaction must've been logged.
+
+     About early returns from the function:
+     - ONE_PHASE option to XA-COMMIT is handled to skip writing XA-commit
+       event now.
+     - check is for the read-only XA that is not to be logged.
+
+     @param thd          THD handle
+     @return error code, 0 success
+  */
+  int write_xa_to_cache(THD *thd);
+
  private:
   bool after_write_to_relay_log(Master_info *mi);
+  /**
+   * Truncte log file and clear LOG_EVENT_BINLOG_IN_USE_F when update is set.
+   * @param[in] log_name name of the log file to be trunacted
+   * @param[in] valid_pos position at which to truncate the log file
+   * @param[in] binlog_size length of the log file before truncated
+   * @param[in] update should the LOG_EVENT_BINLOG_IN_USE_F flag be cleared
+   *                   true - set LOG_EVENT_BINLOG_IN_USE_F to 0
+   *                   false - do not modify LOG_EVENT_BINLOG_IN_USE_F flag
+   * @return true - sucess, false - failed
+   */
+  bool truncate_update_log_file(const char *log_name, my_off_t valid_pos,
+                                my_off_t binlog_size, bool update);
 
  public:
   void make_log_name(char *buf, const char *log_ident);
-  bool is_active(const char *log_file_name);
+  bool is_active(const char *log_file_name) const;
   int remove_logs_from_index(LOG_INFO *linfo, bool need_update_threads);
   int rotate(bool force_rotate, bool *check_purge);
-  void purge();
+
+  /**
+    @brief This function runs automatic purge if the conditions to meet
+    automatic purge are met. Such conditions are: log is open, instance is not
+    locked for backup and automatic purge is enabled.
+
+    If all conditions are met, purge is done according to the configuration
+    of the purge window.
+   */
+  void auto_purge();
+
+  /**
+    @brief This member function is to be called at server startup. It checks if
+    purge can be done and does if it can.
+   */
+  void auto_purge_at_server_startup();
   int rotate_and_purge(THD *thd, bool force_rotate);
 
   bool flush();
@@ -820,6 +940,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
   const char *get_name() const { return name; }
   inline mysql_mutex_t *get_log_lock() { return &LOCK_log; }
   inline mysql_mutex_t *get_commit_lock() { return &LOCK_commit; }
+  inline mysql_mutex_t *get_after_commit_lock() { return &LOCK_after_commit; }
   inline mysql_cond_t *get_log_cond() { return &update_cond; }
   inline Binlog_ofile *get_binlog_file() { return m_binlog_file; }
 
@@ -837,7 +958,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     This function also informs slave about the GTID set sent by the slave,
     transactions missing on the master and few suggestions to recover from
     the error. This message shall be wrapped by
-    ER_MASTER_FATAL_ERROR_READING_BINLOG on slave and will be logged as an
+    ER_SOURCE_FATAL_ERROR_READING_BINLOG on slave and will be logged as an
     error.
 
     This function will be called from mysql_binlog_send() function.
@@ -846,7 +967,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     @param errmsg                      Pointer to the error message
   */
   void report_missing_purged_gtids(const Gtid_set *slave_executed_gtid_set,
-                                   const char **errmsg);
+                                   std::string &errmsg);
 
   /**
     Function to report the missing GTIDs.
@@ -858,7 +979,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
     This function also informs slave about the GTID set sent by the slave,
     transactions missing on the master and few suggestions to recover from
     the error. This message shall be wrapped by
-    ER_MASTER_FATAL_ERROR_READING_BINLOG on slave and will be logged as an
+    ER_SOURCE_FATAL_ERROR_READING_BINLOG on slave and will be logged as an
     error.
 
     This function will be called from find_first_log_not_in_gtid_set()
@@ -870,7 +991,7 @@ class MYSQL_BIN_LOG : public TC_LOG {
   */
   void report_missing_gtids(const Gtid_set *previous_gtid_set,
                             const Gtid_set *slave_executed_gtid_set,
-                            const char **errmsg);
+                            std::string &errmsg);
   static const int MAX_RETRIES_FOR_DELETE_RENAME_FAILURE = 5;
   /*
     It is called by the threads (e.g. dump thread, applier thread) which want
@@ -904,6 +1025,51 @@ class MYSQL_BIN_LOG : public TC_LOG {
     True while rotating binlog, which is caused by logging Incident_log_event.
   */
   bool is_rotating_caused_by_incident;
+
+ public:
+  /**
+    Register LOG_INFO so that log_in_use and adjust_linfo_offsets can
+    operate on all logs. Note that register_log_info, unregister_log_info,
+    log_in_use, adjust_linfo_offsets are is used on global mysql_bin_log object.
+    @param log_info pointer to LOG_INFO which is registred
+  */
+  void register_log_info(LOG_INFO *log_info);
+  /**
+    Unregister LOG_INFO when it is no longer needed.
+    @param log_info pointer to LOG_INFO which is registred
+  */
+  void unregister_log_info(LOG_INFO *log_info);
+  /**
+    Check if any threads use log name.
+    @note This method expects the LOCK_index to be taken so there are no
+    concurrent edits against linfo objects being iterated
+    @param log_name name of a log which is checked for usage
+
+  */
+  int log_in_use(const char *log_name);
+  /**
+    Adjust the position pointer in the binary log file for all running replicas.
+    SYNOPSIS
+      adjust_linfo_offsets()
+      purge_offset Number of bytes removed from start of log index file
+    NOTES
+      - This is called when doing a PURGE when we delete lines from the
+        index log file. This method expects the LOCK_index to be taken so there
+    are no concurrent edits against linfo objects being iterated. REQUIREMENTS
+      - Before calling this function, we have to ensure that no threads are
+        using any binary log file before purge_offset.
+    TODO
+      - Inform the replica threads that they should sync the position
+        in the binary log file with flush_relay_log_info.
+        Now they sync is done for next read.
+  */
+  void adjust_linfo_offsets(my_off_t purge_offset);
+
+ private:
+  mysql_mutex_t LOCK_log_info;
+  // Set of log info objects that are in usage and might prevent some other
+  // operations from executing.
+  std::set<LOG_INFO *> log_info_set;
 };
 
 struct LOAD_FILE_INFO {
@@ -948,7 +1114,7 @@ int check_trx_rw_engines(THD *thd, Transaction_ctx::enum_trx_scope trx_scope);
 */
 bool is_empty_transaction_in_binlog_cache(const THD *thd);
 bool trans_has_updated_trans_table(const THD *thd);
-bool stmt_has_updated_trans_table(Ha_trx_info *ha_list);
+bool stmt_has_updated_trans_table(Ha_trx_info_list const &ha_list);
 bool ending_trans(THD *thd, const bool all);
 bool ending_single_stmt_trans(THD *thd, const bool all);
 bool trans_cannot_safely_rollback(const THD *thd);
@@ -956,8 +1122,19 @@ bool stmt_cannot_safely_rollback(const THD *thd);
 
 int log_loaded_block(IO_CACHE *file);
 
-bool purge_master_logs(THD *thd, const char *to_log);
-bool purge_master_logs_before_date(THD *thd, time_t purge_time);
+/**
+   @brief Purges the binary log files up to the file name passed as
+          a paramenter. Purge will not delete the file passed as
+          an argument.
+
+   @param thd The session context.
+   @param to_log Up to which log file to purge.
+   @return true if there was an error.
+   @return false if there was no error.
+ */
+bool purge_source_logs_to_file(THD *thd, const char *to_log);
+
+bool purge_source_logs_before_date(THD *thd, time_t purge_time);
 bool show_binlog_events(THD *thd, MYSQL_BIN_LOG *binary_log);
 bool mysql_show_binlog_events(THD *thd);
 void check_binlog_cache_size(THD *thd);
