@@ -1,17 +1,18 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,11 +38,11 @@
 #include "my_psi_config.h"
 #include "my_sys.h"
 #include "my_systime.h"  // Timout_type
-#include "mysql/components/services/mysql_cond_bits.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/mysql_rwlock_bits.h"
-#include "mysql/components/services/psi_mdl_bits.h"
-#include "mysql/components/services/psi_stage_bits.h"
+#include "mysql/components/services/bits/mysql_cond_bits.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
+#include "mysql/components/services/bits/mysql_rwlock_bits.h"
+#include "mysql/components/services/bits/psi_mdl_bits.h"
+#include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql_com.h"
 #include "sql/sql_plist.h"
@@ -83,7 +84,7 @@ bool test_drive_fix_pins(MDL_context *);
 
 class MDL_context_owner {
  public:
-  virtual ~MDL_context_owner() {}
+  virtual ~MDL_context_owner() = default;
 
   /**
     Enter a condition wait.
@@ -125,7 +126,7 @@ class MDL_context_owner {
   /**
     Does the owner still have connection to the client?
   */
-  virtual bool is_connected() = 0;
+  virtual bool is_connected(bool = false) = 0;
 
   /**
     Indicates that owner thread might have some commit order (non-MDL) waits
@@ -608,23 +609,25 @@ struct MDL_key {
 
     @remark The key for a routine/event/resource group/trigger is
       @<mdl_namespace@>+@<database name@>+@<normalized object name@>
-      additionaly @<object name@> is stored in the same buffer for information
-      purpose if buffer has sufficent space.
+      additionally @<object name@> is stored in the same buffer for information
+      purpose if buffer has sufficient space.
 
     Routine, Event and Resource group names are case sensitive and accent
     sensitive. So normalized object name is used to form a MDL_key.
 
-    With the UTF8MB3 charset space reserved for the db name/object name is
-    64 * 3  bytes. utf8_general_ci collation is used for the Routine, Event and
-    Resource group names. With this collation, the normalized object name uses
-    just 2 bytes for each character (max length = 64 * 2 bytes). MDL_key has
-    still some space to store the object names. If there is a sufficient space
-    for the object name in the MDL_key then it is stored in the MDL_key (similar
-    to the column names in the MDL_key). Actual object name is used by the PFS.
-    Not listing actual object name from the PFS should be OK when there is no
-    space to store it (instead of increasing the MDL_key size). Object name is
-    not used in the key comparisons. So only (mdl_namespace + strlen(db) + 1 +
-    normalized_name_len + 1) value is stored in the m_length member.
+    With the UTF8MB3 charset space reserved for the db name/object
+    name is 64 * 3 bytes. utf8mb3_general_ci collation is used for the
+    Routine, Event and Resource group names. With this collation, the
+    normalized object name uses just 2 bytes for each character (max
+    length = 64 * 2 bytes). MDL_key has still some space to store the
+    object names. If there is a sufficient space for the object name
+    in the MDL_key then it is stored in the MDL_key (similar to the
+    column names in the MDL_key). Actual object name is used by the
+    PFS.  Not listing actual object name from the PFS should be OK
+    when there is no space to store it (instead of increasing the
+    MDL_key size). Object name is not used in the key comparisons. So
+    only (mdl_namespace + strlen(db) + 1 + normalized_name_len + 1)
+    value is stored in the m_length member.
 
     @param  mdl_namespace       Id of namespace of object to be locked.
     @param  db                  Name of database to which the object belongs.
@@ -754,7 +757,7 @@ struct MDL_key {
           const char *name_arg) {
     mdl_key_init(namespace_arg, db_arg, name_arg);
   }
-  MDL_key() {} /* To use when part of MDL_request. */
+  MDL_key() = default; /* To use when part of MDL_request. */
 
   /**
     Get thread state name to be used in case when we have to
@@ -821,8 +824,8 @@ class MDL_request {
 
  public:
   static void *operator new(size_t size, MEM_ROOT *mem_root,
-                            const std::nothrow_t &arg MY_ATTRIBUTE((unused)) =
-                                std::nothrow) noexcept {
+                            const std::nothrow_t &arg
+                            [[maybe_unused]] = std::nothrow) noexcept {
     return mem_root->Alloc(size);
   }
 
@@ -870,14 +873,15 @@ class MDL_request {
   /**
     This constructor exists for two reasons:
 
-    - TABLE_LIST objects are sometimes default-constructed. We plan to remove
-      this as there is no practical reason, the call to the default
-      constructor is always followed by either a call to TABLE_LIST::operator=
-      or memberwise assignments.
+    - Table_ref objects are sometimes default-constructed. We plan to
+      remove this as there is no practical reason, the call to the default
+      constructor is always followed by either a call to
+      Table_ref::operator= or memberwise assignments.
 
-    - In some legacy cases TABLE_LIST objects are copy-assigned without
-      intention to copy the TABLE_LIST::mdl_request member. In this cases they
-      are overwritten with an uninitialized MDL_request object. The cases are:
+    - In some legacy cases Table_ref objects are copy-assigned without
+      intention to copy the Table_ref::mdl_request member. In this cases
+      they are overwritten with an uninitialized MDL_request object. The cases
+      are:
 
       - Sql_cmd_handler_open::execute()
       - mysql_execute_command()
@@ -888,7 +892,7 @@ class MDL_request {
       Locked_tables_list::rename_locked_table(), a move assignment is actually
       what is intended.
   */
-  MDL_request() {}
+  MDL_request() = default;
 
   MDL_request(const MDL_request &rhs)
       : type(rhs.type), duration(rhs.duration), ticket(nullptr), key(rhs.key) {}
@@ -1311,7 +1315,7 @@ class MDL_ticket_store {
 
 class MDL_savepoint {
  public:
-  MDL_savepoint() {}
+  MDL_savepoint() = default;
 
  private:
   MDL_savepoint(MDL_ticket *stmt_ticket, MDL_ticket *trans_ticket)
@@ -1374,7 +1378,7 @@ class MDL_wait {
 
 class MDL_release_locks_visitor {
  public:
-  virtual ~MDL_release_locks_visitor() {}
+  virtual ~MDL_release_locks_visitor() = default;
   /**
     Check if the given ticket represents a lock that should be released.
 
@@ -1389,7 +1393,7 @@ class MDL_release_locks_visitor {
 
 class MDL_context_visitor {
  public:
-  virtual ~MDL_context_visitor() {}
+  virtual ~MDL_context_visitor() = default;
   virtual void visit_context(const MDL_context *ctx) = 0;
 };
 
@@ -1739,37 +1743,4 @@ const double MDL_LOCKS_UNUSED_LOCKS_MIN_RATIO = 0.25;
 
 int32 mdl_get_unused_locks_count();
 
-/**
-  Inspect if MDL_context is owned by any thread.
-*/
-class MDL_lock_is_owned_visitor : public MDL_context_visitor {
- public:
-  MDL_lock_is_owned_visitor() : m_exists(false) {}
-
-  /**
-    Collects relevant information about the MDL lock owner.
-
-    This function is only called by MDL_context::find_lock_owner() when
-    searching for MDL lock owners to collect extra information about the
-    owner. As we only need to know that the MDL lock is owned, setting
-    m_exists to true is enough.
-  */
-
-  void visit_context(const MDL_context *ctx MY_ATTRIBUTE((unused))) override {
-    m_exists = true;
-  }
-
-  /**
-    Returns if an owner for the MDL lock being inspected exists.
-
-    @return true when MDL lock is owned, false otherwise.
-  */
-
-  bool exists() const { return m_exists; }
-
- private:
-  /* holds information about MDL being owned by any thread */
-  bool m_exists;
-};
-
 #endif
