@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -629,23 +630,23 @@ err:
   switch (errpos) {
     case 6:
       my_free(m_info);
-      /* fall through */
+      [[fallthrough]];
     case 5:
       (void)mysql_file_close(info.dfile, MYF(0));
       if (old_share) break; /* Don't remove open table */
 
-      /* fall through */
+      [[fallthrough]];
     case 4:
       my_free(share);
-      /* fall through */
+      [[fallthrough]];
     case 3:
       if (!lock_error) (void)my_lock(kfile, F_UNLCK, MYF(MY_SEEK_NOT_DONE));
-      /* fall through */
+      [[fallthrough]];
     case 2:
-      /* fall through */
+      [[fallthrough]];
     case 1:
       (void)mysql_file_close(kfile, MYF(0));
-      /* fall through */
+      [[fallthrough]];
     case 0:
     default:
       break;
@@ -761,7 +762,7 @@ static void setup_key_functions(MI_KEYDEF *keyinfo) {
         _mi_prefix_search() compares end-space against ASCII blank (' ').
         It cannot be used for character sets, that do not encode the
         blank character like ASCII does. UCS2 is an example. All
-        character sets with a fixed width > 1 or a mimimum width > 1
+        character sets with a fixed width > 1 or a minimum width > 1
         cannot represent blank like ASCII does. In these cases we have
         to use _mi_seq_search() for the search.
       */
@@ -1227,7 +1228,7 @@ exist a dup()-like call that would give us two different file descriptors.
 *************************************************************************/
 
 int mi_open_datafile(MI_INFO *info, MYISAM_SHARE *share, const char *org_name,
-                     File file_to_dup MY_ATTRIBUTE((unused))) {
+                     File file_to_dup [[maybe_unused]]) {
   char *data_name = share->data_file_name;
   char real_data_name[FN_REFLEN];
   ST_FILE_ID file_id = {0, 0};
