@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +30,6 @@
 
 #define JAM_FILE_ID 57
 
-
 class TupKeyReq {
   /**
    * Reciver(s)
@@ -44,20 +44,19 @@ class TupKeyReq {
   /**
    * For printing
    */
-  friend bool printTUPKEYREQ(FILE * output, const Uint32 * theData, Uint32 len, Uint16 receiverBlockNo);
+  friend bool printTUPKEYREQ(FILE *output, const Uint32 *theData, Uint32 len,
+                             Uint16 receiverBlockNo);
 
-public:
-  STATIC_CONST( SignalLength = 10 );
+ public:
+  static constexpr Uint32 SignalLength = 10;
 
-  enum
-  {
+  enum {
     OP_PRIMARY_REPLICA = 0,
     OP_BACKUP_REPLICA = 1,
     OP_NO_TRIGGERS = 2,
   };
 
-private:
-
+ private:
   /**
    * DATA VARIABLES
    */
@@ -72,10 +71,10 @@ private:
   Uint32 m_row_id_page_idx;
   Uint32 attrInfoIVal;
 
-  static Uint32 getInterpretedFlag(Uint32 const& requestInfo);
-  static Uint32 getRowidFlag(Uint32 const& requestInfo);
-  static void setInterpretedFlag(Uint32 & requestInfo, Uint32 value);
-  static void setRowidFlag(Uint32 & requestInfo, Uint32 value);
+  static Uint32 getInterpretedFlag(Uint32 const &requestInfo);
+  static Uint32 getRowidFlag(Uint32 const &requestInfo);
+  static void setInterpretedFlag(Uint32 &requestInfo, Uint32 value);
+  static void setRowidFlag(Uint32 &requestInfo, Uint32 value);
 
   /*
     Request Info
@@ -86,34 +85,28 @@ private:
   */
 
   enum RequestInfo {
-    INTERPRETED_POS = 10, INTERPRETED_MASK = 1,
-    ROWID_POS       = 11, ROWID_MASK       = 1
+    INTERPRETED_POS = 10,
+    INTERPRETED_MASK = 1,
+    ROWID_POS = 11,
+    ROWID_MASK = 1
   };
 };
 
-inline Uint32
-TupKeyReq::getInterpretedFlag(Uint32 const& requestInfo)
-{
+inline Uint32 TupKeyReq::getInterpretedFlag(Uint32 const &requestInfo) {
   return (requestInfo >> INTERPRETED_POS) & INTERPRETED_MASK;
 }
 
-inline Uint32
-TupKeyReq::getRowidFlag(Uint32 const& requestInfo)
-{
+inline Uint32 TupKeyReq::getRowidFlag(Uint32 const &requestInfo) {
   return (requestInfo >> ROWID_POS) & ROWID_MASK;
 }
 
-inline void
-TupKeyReq::setInterpretedFlag(Uint32 & requestInfo, Uint32 value)
-{
+inline void TupKeyReq::setInterpretedFlag(Uint32 &requestInfo, Uint32 value) {
   assert(value <= INTERPRETED_MASK);
   assert((requestInfo & (INTERPRETED_MASK << INTERPRETED_POS)) == 0);
   requestInfo |= value << INTERPRETED_POS;
 }
 
-inline void
-TupKeyReq::setRowidFlag(Uint32 & requestInfo, Uint32 value)
-{
+inline void TupKeyReq::setRowidFlag(Uint32 &requestInfo, Uint32 value) {
   assert(value <= ROWID_MASK);
   assert((requestInfo & (ROWID_MASK << ROWID_POS)) == 0);
   requestInfo |= value << ROWID_POS;
@@ -133,13 +126,13 @@ class TupKeyConf {
   /**
    * For printing
    */
-  friend bool printTUPKEYCONF(FILE * output, const Uint32 * theData, Uint32 len, Uint16 receiverBlockNo);
+  friend bool printTUPKEYCONF(FILE *output, const Uint32 *theData, Uint32 len,
+                              Uint16 receiverBlockNo);
 
-public:
-  STATIC_CONST( SignalLength = 7 );
-
-private:
+ public:
+  static constexpr Uint32 SignalLength = 7;
 
+ private:
   /**
    * DATA VARIABLES
    */
@@ -157,7 +150,7 @@ class TupKeyRef {
   /**
    * Reciver(s)
    */
-  friend class Dblqh;      
+  friend class Dblqh;
 
   /**
    * Sender(s)
@@ -167,13 +160,13 @@ class TupKeyRef {
   /**
    * For printing
    */
-  friend bool printTUPKEYREF(FILE * output, const Uint32 * theData, Uint32 len, Uint16 receiverBlockNo);
+  friend bool printTUPKEYREF(FILE *output, const Uint32 *theData, Uint32 len,
+                             Uint16 receiverBlockNo);
 
-public:
-  STATIC_CONST( SignalLength = 3 );
-
-private:
+ public:
+  static constexpr Uint32 SignalLength = 3;
 
+ private:
   /**
    * DATA VARIABLES
    */
@@ -183,7 +176,6 @@ private:
   Uint32 noExecInstructions;
 };
 
-
 #undef JAM_FILE_ID
 
 #endif
