@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -226,7 +227,7 @@ int init_instruments(const PFS_global_param *param) {
 }
 
 /** Cleanup all the instruments buffers. */
-void cleanup_instruments(void) {
+void cleanup_instruments() {
   global_mutex_container.cleanup();
   global_rwlock_container.cleanup();
   global_cond_container.cleanup();
@@ -266,26 +267,64 @@ static const uchar *filename_hash_get_key(const uchar *entry, size_t *length) {
   assert(typed_entry != nullptr);
   file = *typed_entry;
   assert(file != nullptr);
-  *length = file->m_filename_length;
-  result = file->m_filename;
+  *length = sizeof(file->m_file_name);
+  result = &file->m_file_name;
   return reinterpret_cast<const uchar *>(result);
 }
 
+static uint filename_hash_func(const LF_HASH *, const uchar *key,
+                               size_t key_len [[maybe_unused]]) {
+  const PFS_file_name *file_name_key;
+  uint64 nr1;
+  uint64 nr2;
+
+  assert(key_len == sizeof(PFS_file_name));
+  file_name_key = reinterpret_cast<const PFS_file_name *>(key);
+  assert(file_name_key != nullptr);
+
+  nr1 = 0;
+  nr2 = 0;
+
+  file_name_key->hash(&nr1, &nr2);
+
+  return nr1;
+}
+
+static int filename_hash_cmp_func(const uchar *key1,
+                                  size_t key_len1 [[maybe_unused]],
+                                  const uchar *key2,
+                                  size_t key_len2 [[maybe_unused]]) {
+  const PFS_file_name *file_name_key1;
+  const PFS_file_name *file_name_key2;
+  int cmp;
+
+  assert(key_len1 == sizeof(PFS_file_name));
+  assert(key_len2 == sizeof(PFS_file_name));
+  file_name_key1 = reinterpret_cast<const PFS_file_name *>(key1);
+  file_name_key2 = reinterpret_cast<const PFS_file_name *>(key2);
+  assert(file_name_key1 != nullptr);
+  assert(file_name_key2 != nullptr);
+
+  cmp = file_name_key1->sort(file_name_key2);
+  return cmp;
+}
+
 /**
   Initialize the file name hash.
   @return 0 on success
 */
 int init_file_hash(const PFS_global_param *param) {
   if ((!filename_hash_inited) && (param->m_file_sizing != 0)) {
-    lf_hash_init(&filename_hash, sizeof(PFS_file *), LF_HASH_UNIQUE, 0, 0,
-                 filename_hash_get_key, &my_charset_bin);
+    lf_hash_init3(&filename_hash, sizeof(PFS_file *), LF_HASH_UNIQUE,
+                  filename_hash_get_key, filename_hash_func,
+                  filename_hash_cmp_func, nullptr, nullptr, nullptr);
     filename_hash_inited = true;
   }
   return 0;
 }
 
 /** Cleanup the file name hash. */
-void cleanup_file_hash(void) {
+void cleanup_file_hash() {
   if (filename_hash_inited) {
     lf_hash_destroy(&filename_hash);
     filename_hash_inited = false;
@@ -545,17 +584,30 @@ void carry_global_memory_stat_free_delta(PFS_memory_stat_free_delta *delta,
   (void)stat->apply_free_delta(delta, &delta_buffer);
 }
 
+void PFS_thread::mem_cnt_alloc(size_t size) {
+#ifndef NDEBUG
+  thd_mem_cnt_alloc(m_cnt_thd, size, current_key_name);
+#else
+  thd_mem_cnt_alloc(m_cnt_thd, size);
+#endif
+}
+
+void PFS_thread::mem_cnt_free(size_t size) {
+  thd_mem_cnt_free(m_cnt_thd, size);
+}
+
 /**
   Create instrumentation for a thread instance.
   @param klass                        the thread class
+  @param seqnum                       the thread instance sequence number
   @param identity                     the thread address,
     or a value characteristic of this thread
   @param processlist_id               the PROCESSLIST id,
     or 0 if unknown
   @return a thread instance, or NULL
 */
-PFS_thread *create_thread(PFS_thread_class *klass,
-                          const void *identity MY_ATTRIBUTE((unused)),
+PFS_thread *create_thread(PFS_thread_class *klass, PSI_thread_seqnum seqnum,
+                          const void *identity [[maybe_unused]],
                           ulonglong processlist_id) {
   PFS_thread *pfs;
   pfs_dirty_state dirty_state;
@@ -596,9 +648,9 @@ PFS_thread *create_thread(PFS_thread_class *klass,
     pfs->m_digest_hash_pins = nullptr;
     pfs->m_program_hash_pins = nullptr;
 
-    pfs->m_username_length = 0;
-    pfs->m_hostname_length = 0;
-    pfs->m_dbname_length = 0;
+    pfs->m_user_name.reset();
+    pfs->m_host_name.reset();
+    pfs->m_db_name.reset();
     pfs->m_groupname_length = 0;
     pfs->m_user_data = nullptr;
     pfs->m_command = 0;
@@ -607,9 +659,14 @@ PFS_thread *create_thread(PFS_thread_class *klass,
     pfs->m_stage_progress = nullptr;
     pfs->m_processlist_info[0] = '\0';
     pfs->m_processlist_info_length = 0;
+    pfs->m_secondary = false;
     pfs->m_connection_type = NO_VIO_TYPE;
 
     pfs->m_thd = nullptr;
+    pfs->m_cnt_thd = nullptr;
+#ifndef NDEBUG
+    pfs->current_key_name = nullptr;
+#endif
     pfs->m_host = nullptr;
     pfs->m_user = nullptr;
     pfs->m_account = nullptr;
@@ -640,6 +697,35 @@ PFS_thread *create_thread(PFS_thread_class *klass,
     pfs->m_events_statements_count = 0;
     pfs->m_transaction_current.m_event_id = 0;
 
+    if (klass->is_singleton()) {
+#if 0
+      /* See destroy_thread() */
+      assert(klass->m_singleton == nullptr);
+#endif
+      klass->m_singleton = pfs;
+    }
+
+    if (klass->has_seqnum()) {
+      if (klass->has_auto_seqnum()) {
+        seqnum = klass->m_seqnum++;
+      }
+
+      /* Possible truncation */
+      (void)snprintf(pfs->m_os_name, PFS_MAX_OS_NAME_LENGTH - 1,
+                     klass->m_os_name, seqnum);
+    } else {
+      (void)snprintf(pfs->m_os_name, PFS_MAX_OS_NAME_LENGTH, "%s",
+                     klass->m_os_name);
+    }
+    pfs->m_os_name[PFS_MAX_OS_NAME_LENGTH - 1] = '\0';
+
+    pfs->m_session_all_memory_stat.reset();
+
+#ifdef HAVE_PSI_SERVER_TELEMETRY_TRACES_INTERFACE
+    pfs->m_telemetry = nullptr;
+    pfs->m_telemetry_session = nullptr;
+#endif /* HAVE_PSI_SERVER_TELEMETRY_TRACES_INTERFACE */
+
     pfs->m_lock.dirty_to_allocated(&dirty_state);
   }
 
@@ -740,6 +826,54 @@ void destroy_thread(PFS_thread *pfs) {
   assert(pfs != nullptr);
   pfs->reset_session_connect_attrs();
   pfs->m_thd = nullptr;
+  pfs->m_cnt_thd = nullptr;
+
+#ifdef HAVE_PSI_SERVER_TELEMETRY_TRACES_INTERFACE
+  if (pfs->m_telemetry_session != nullptr) {
+    assert(pfs->m_telemetry != nullptr);
+    pfs->m_telemetry->m_tel_session_destroy(pfs->m_telemetry_session);
+  }
+
+  pfs->m_telemetry = nullptr;
+  pfs->m_telemetry_session = nullptr;
+#endif /* HAVE_PSI_SERVER_TELEMETRY_TRACES_INTERFACE */
+
+  PFS_thread_class *klass = pfs->m_class;
+  if (klass->is_singleton()) {
+#if 0
+    /*
+      In theory, some threads are -- logically -- singletons.
+
+      START XYZ will start a XYZ thread with mysql_thread_create(),
+      and set a global state as "running".
+      STOP XYZ will notify the running thread to stop,
+      and set a global state as "not running".
+
+      In practice, these threads are -- physically -- not singletons.
+
+      STOP XYZ only notifies the running thread to stop,
+      but the MySQL code base in general never calls pthread_join(3).
+      Instead, the running thread, when noticing it should stop,
+      is still executing cleanup actions, until the thread main loop
+      finally terminates and call destroy_thread in the performance schema.
+
+      Because of this, the following sequence:
+        START XYZ --> fork thread #1
+        STOP XYZ --> tell #1 to stop
+        START XYZ --> fork thread #2
+      can create situations when both threads #1 and #2
+      are executing at the same time, until #1 gracefully ends.
+
+      As a result, the assert below is relaxed.
+
+      To enforce it properly,
+      a pre requisite is first to use pthread_join() in the entire code base.
+    */
+
+    assert(klass->m_singleton == pfs);
+#endif
+    klass->m_singleton = nullptr;
+  }
 
   if (pfs->m_account != nullptr) {
     pfs->m_account->release();
@@ -814,18 +948,15 @@ static LF_PINS *get_filename_hash_pins(PFS_thread *thread) {
   Normalize a filename with fully qualified path.
   @param filename                 file to be normalized, null-terminatd
   @param name_len                 length in bytes of the filename
-  @param buffer                   output buffer
-  @param buffer_len               size in bytes of buffer, must be >= FN_REFLEN
-  @return normalized, null-terminated filename or NULL
+  @param [out] normalized Normalized file name
 */
-char *normalize_filename(const char *filename, uint name_len, char *buffer,
-                         uint buffer_len) {
+int normalize_filename(const char *filename, uint name_len,
+                       PFS_file_name &normalized) {
+  char buffer[FN_REFLEN];
   char safe_buffer[FN_REFLEN];
   const char *safe_filename;
 
-  assert(filename != NULL);
-  assert(buffer != NULL);
-  assert(buffer_len >= FN_REFLEN);
+  assert(filename != nullptr);
 
   if (name_len >= FN_REFLEN) {
     /*
@@ -867,7 +998,7 @@ char *normalize_filename(const char *filename, uint name_len, char *buffer,
     works properly for files that do not exist (yet) on the file system.
   */
   char dirbuffer[FN_REFLEN + 1];
-  size_t dirlen = dirname_length(safe_filename);
+  const size_t dirlen = dirname_length(safe_filename);
 
   if (dirlen == 0) {
     dirbuffer[0] = FN_CURLIB;
@@ -881,12 +1012,12 @@ char *normalize_filename(const char *filename, uint name_len, char *buffer,
   /* Resolve the absolute directory path. */
   if (my_realpath(buffer, dirbuffer, MYF(0)) != 0) {
     buffer[0] = '\0';
-    return nullptr;
+    return 1;
   }
 
   /* Append the unresolved filename to the resolved path */
   char *ptr = buffer + strlen(buffer);
-  char *buf_end = &buffer[buffer_len - 1];
+  char *buf_end = &buffer[sizeof(buffer) - 1];
   if ((buf_end > ptr) && (*(ptr - 1) != FN_LIBCHAR)) {
     *ptr++ = FN_LIBCHAR;
   }
@@ -896,7 +1027,8 @@ char *normalize_filename(const char *filename, uint name_len, char *buffer,
   *buf_end = '\0';
 
   /* Return normalized filename. */
-  return buffer;
+  normalized.set(buffer, strlen(buffer));
+  return 0;
 }
 
 /**
@@ -913,17 +1045,14 @@ PFS_file *find_or_create_file(PFS_thread *thread, PFS_file_class *klass,
   assert(klass != nullptr || !create);
 
   PFS_file *pfs;
-  char buffer[FN_REFLEN];
-  const char *normalized_filename;
-  uint normalized_length;
+  PFS_file_name normalized_filename;
+  int rc;
 
-  normalized_filename =
-      normalize_filename(filename, len, buffer, (uint)sizeof(buffer));
-  if (normalized_filename == nullptr) {
+  rc = normalize_filename(filename, len, normalized_filename);
+  if (rc != 0) {
     global_file_container.m_lost++;
     return nullptr;
   }
-  normalized_length = (uint)strlen(normalized_filename);
 
   LF_PINS *pins = get_filename_hash_pins(thread);
   if (unlikely(pins == nullptr)) {
@@ -939,7 +1068,7 @@ PFS_file *find_or_create_file(PFS_thread *thread, PFS_file_class *klass,
 search:
 
   entry = reinterpret_cast<PFS_file **>(lf_hash_search(
-      &filename_hash, pins, normalized_filename, normalized_length));
+      &filename_hash, pins, &normalized_filename, sizeof(normalized_filename)));
   if (entry && (entry != MY_LF_ERRPTR)) {
     pfs = *entry;
     pfs->m_file_stat.m_open_count++;
@@ -959,9 +1088,7 @@ search:
     pfs->m_class = klass;
     pfs->m_enabled = klass->m_enabled && flag_global_instrumentation;
     pfs->m_timed = klass->m_timed;
-    memcpy(pfs->m_filename, normalized_filename, normalized_length);
-    pfs->m_filename[normalized_length] = '\0';
-    pfs->m_filename_length = normalized_length;
+    pfs->m_file_name = normalized_filename;
     pfs->m_file_stat.m_open_count = 1;
     pfs->m_file_stat.m_io_stat.reset();
     pfs->m_identity = (const void *)pfs;
@@ -1008,18 +1135,15 @@ PFS_file *start_file_rename(PFS_thread *thread, const char *old_name) {
   assert(thread != nullptr);
   assert(old_name != nullptr);
 
-  uint old_length = (uint)strlen(old_name);
-  char buffer[FN_REFLEN];
-  const char *normalized_filename;
-  uint normalized_length;
+  const uint old_length = (uint)strlen(old_name);
+  PFS_file_name normalized_filename;
+  int rc;
 
-  normalized_filename =
-      normalize_filename(old_name, old_length, buffer, (uint)sizeof(buffer));
-  if (normalized_filename == nullptr) {
+  rc = normalize_filename(old_name, old_length, normalized_filename);
+  if (rc != 0) {
     global_file_container.m_lost++;
     return nullptr;
   }
-  normalized_length = (uint)strlen(normalized_filename);
 
   LF_PINS *pins = get_filename_hash_pins(thread);
   if (unlikely(pins == nullptr)) {
@@ -1028,8 +1152,8 @@ PFS_file *start_file_rename(PFS_thread *thread, const char *old_name) {
   }
 
   /* Find the file instrumentation by name. */
-  PFS_file **entry = reinterpret_cast<PFS_file **>(lf_hash_search(
-      &filename_hash, pins, normalized_filename, normalized_length));
+  auto **entry = reinterpret_cast<PFS_file **>(lf_hash_search(
+      &filename_hash, pins, &normalized_filename, sizeof(normalized_filename)));
 
   PFS_file *pfs = nullptr;
   if (entry && (entry != MY_LF_ERRPTR)) {
@@ -1038,8 +1162,8 @@ PFS_file *start_file_rename(PFS_thread *thread, const char *old_name) {
       Add the new filename after rename() operation completes.
     */
     pfs = *entry;
-    lf_hash_delete(&filename_hash, pins, pfs->m_filename,
-                   pfs->m_filename_length);
+    lf_hash_delete(&filename_hash, pins, &pfs->m_file_name,
+                   sizeof(pfs->m_file_name));
   }
 
   lf_hash_search_unpin(pins);
@@ -1061,40 +1185,23 @@ int end_file_rename(PFS_thread *thread, PFS_file *pfs, const char *new_name,
   assert(pfs != nullptr);
   assert(new_name != nullptr);
 
-  uint new_length = (uint)strlen(new_name);
-  const char *filename;
-  uint name_length;
+  const uint new_length = (uint)strlen(new_name);
 
-  /*
-    If rename() succeeded, than add the new filename to the hash,
-    othwerise restore the old filename.
-  */
   if (likely(rename_result == 0)) {
-    filename = new_name;
-    name_length = new_length;
-  } else {
-    filename = pfs->m_filename;
-    name_length = pfs->m_filename_length;
-  }
+    /*
+      If rename() succeeded,
+      update the file instance with the new filename.
+    */
+    PFS_file_name normalized_filename;
+    int rc;
 
-  char buffer[FN_REFLEN];
-  const char *normalized_filename;
-  uint normalized_length;
+    rc = normalize_filename(new_name, new_length, normalized_filename);
+    if (rc != 0) {
+      global_file_container.m_lost++;
+      return 1;
+    }
 
-  normalized_filename =
-      normalize_filename(filename, name_length, buffer, (uint)sizeof(buffer));
-  if (normalized_filename == nullptr) {
-    global_file_container.m_lost++;
-    return 1;
-  }
-
-  normalized_length = (uint)strlen(normalized_filename);
-
-  /* Update the file instance with the new filename. */
-  if (rename_result == 0) {
-    memcpy(pfs->m_filename, normalized_filename, normalized_length);
-    pfs->m_filename[normalized_length] = '\0';
-    pfs->m_filename_length = normalized_length;
+    pfs->m_file_name = normalized_filename;
   }
 
   LF_PINS *pins = get_filename_hash_pins(thread);
@@ -1105,7 +1212,7 @@ int end_file_rename(PFS_thread *thread, PFS_file *pfs, const char *new_name,
   }
 
   /* Add the new filename or restore the old filename to the hash. */
-  int res = lf_hash_insert(&filename_hash, pins, &pfs);
+  const int res = lf_hash_insert(&filename_hash, pins, &pfs);
   lf_hash_search_unpin(pins);
 
   if (unlikely(res != 0)) {
@@ -1149,7 +1256,8 @@ void delete_file_name(PFS_thread *thread, PFS_file *pfs) {
   LF_PINS *pins = get_filename_hash_pins(thread);
   assert(pins != nullptr);
 
-  lf_hash_delete(&filename_hash, pins, pfs->m_filename, pfs->m_filename_length);
+  lf_hash_delete(&filename_hash, pins, &pfs->m_file_name,
+                 sizeof(pfs->m_file_name));
 }
 
 /**
@@ -1214,7 +1322,7 @@ PFS_table *create_table(PFS_table_share *share, PFS_thread *opening_thread,
   return pfs;
 }
 
-void PFS_table::sanitized_aggregate(void) {
+void PFS_table::sanitized_aggregate() {
   /*
     This thread could be a TRUNCATE on an aggregated summary table,
     and not own the table handle.
@@ -1232,7 +1340,7 @@ void PFS_table::sanitized_aggregate(void) {
   }
 }
 
-void PFS_table::sanitized_aggregate_io(void) {
+void PFS_table::sanitized_aggregate_io() {
   PFS_table_share *safe_share = sanitize_table_share(m_share);
   if (safe_share != nullptr && m_has_io_stats) {
     safe_aggregate_io(nullptr, &m_table_stat, safe_share);
@@ -1240,7 +1348,7 @@ void PFS_table::sanitized_aggregate_io(void) {
   }
 }
 
-void PFS_table::sanitized_aggregate_lock(void) {
+void PFS_table::sanitized_aggregate_lock() {
   PFS_table_share *safe_share = sanitize_table_share(m_share);
   if (safe_share != nullptr && m_has_lock_stats) {
     safe_aggregate_lock(&m_table_stat, safe_share);
@@ -1254,7 +1362,7 @@ void PFS_table::safe_aggregate_io(const TABLE_SHARE *optional_server_share,
   assert(table_stat != nullptr);
   assert(table_share != nullptr);
 
-  uint key_count = sanitize_index_count(table_share->m_key_count);
+  const uint key_count = sanitize_index_count(table_share->m_key_count);
 
   PFS_table_share_index *to_stat;
   PFS_table_io_stat *from_stat;
@@ -1411,7 +1519,7 @@ void destroy_socket(PFS_socket *pfs) {
     if (stat.m_count != 0) {
       PFS_single_stat *event_name_array;
       event_name_array = thread->write_instr_class_waits_stats();
-      uint index = pfs->m_class->m_event_name_index;
+      const uint index = pfs->m_class->m_event_name_index;
 
       event_name_array[index].aggregate(&stat);
     }
@@ -1460,7 +1568,7 @@ void destroy_metadata_lock(PFS_metadata_lock *pfs) {
 
 static void fct_reset_mutex_waits(PFS_mutex *pfs) { pfs->m_mutex_stat.reset(); }
 
-static void reset_mutex_waits_by_instance(void) {
+static void reset_mutex_waits_by_instance() {
   global_mutex_container.apply_all(fct_reset_mutex_waits);
 }
 
@@ -1468,19 +1576,19 @@ static void fct_reset_rwlock_waits(PFS_rwlock *pfs) {
   pfs->m_rwlock_stat.reset();
 }
 
-static void reset_rwlock_waits_by_instance(void) {
+static void reset_rwlock_waits_by_instance() {
   global_rwlock_container.apply_all(fct_reset_rwlock_waits);
 }
 
 static void fct_reset_cond_waits(PFS_cond *pfs) { pfs->m_cond_stat.reset(); }
 
-static void reset_cond_waits_by_instance(void) {
+static void reset_cond_waits_by_instance() {
   global_cond_container.apply_all(fct_reset_cond_waits);
 }
 
 static void fct_reset_file_waits(PFS_file *pfs) { pfs->m_file_stat.reset(); }
 
-static void reset_file_waits_by_instance(void) {
+static void reset_file_waits_by_instance() {
   global_file_container.apply_all(fct_reset_file_waits);
 }
 
@@ -1488,12 +1596,12 @@ static void fct_reset_socket_waits(PFS_socket *pfs) {
   pfs->m_socket_stat.reset();
 }
 
-static void reset_socket_waits_by_instance(void) {
+static void reset_socket_waits_by_instance() {
   global_socket_container.apply_all(fct_reset_socket_waits);
 }
 
 /** Reset the wait statistics per object instance. */
-void reset_events_waits_by_instance(void) {
+void reset_events_waits_by_instance() {
   reset_mutex_waits_by_instance();
   reset_rwlock_waits_by_instance();
   reset_cond_waits_by_instance();
@@ -1506,7 +1614,7 @@ static void fct_reset_file_io(PFS_file *pfs) {
 }
 
 /** Reset the I/O statistics per file instance. */
-void reset_file_instance_io(void) {
+void reset_file_instance_io() {
   global_file_container.apply_all(fct_reset_file_io);
 }
 
@@ -1515,7 +1623,7 @@ static void fct_reset_socket_io(PFS_socket *pfs) {
 }
 
 /** Reset the I/O statistics per socket instance. */
-void reset_socket_instance_io(void) {
+void reset_socket_instance_io() {
   global_socket_container.apply_all(fct_reset_socket_io);
 }
 
@@ -1843,26 +1951,30 @@ void aggregate_thread_status(PFS_thread *thread, PFS_account *safe_account,
   if (safe_host != nullptr) {
     safe_host->aggregate_status_stats(status_var);
   }
-
-  return;
 }
 
-static void aggregate_thread_stats(PFS_thread *, PFS_account *safe_account,
+static void aggregate_thread_stats(PFS_thread *thread,
+                                   PFS_account *safe_account,
                                    PFS_user *safe_user, PFS_host *safe_host) {
+  const ulonglong controlled_memory =
+      thread->m_session_all_memory_stat.m_controlled.get_session_max();
+  const ulonglong total_memory =
+      thread->m_session_all_memory_stat.m_total.get_session_max();
+
   if (likely(safe_account != nullptr)) {
-    safe_account->m_disconnected_count++;
+    safe_account->aggregate_disconnect(controlled_memory, total_memory);
+    return;
   }
 
   if (safe_user != nullptr) {
-    safe_user->m_disconnected_count++;
+    safe_user->aggregate_disconnect(controlled_memory, total_memory);
   }
 
   if (safe_host != nullptr) {
-    safe_host->m_disconnected_count++;
+    safe_host->aggregate_disconnect(controlled_memory, total_memory);
   }
 
   /* There is no global table for connections statistics. */
-  return;
 }
 
 void aggregate_thread(PFS_thread *thread, PFS_account *safe_account,
@@ -2289,17 +2401,14 @@ void set_thread_account(PFS_thread *thread) {
   assert(thread->m_user == nullptr);
   assert(thread->m_host == nullptr);
 
-  thread->m_account = find_or_create_account(
-      thread, thread->m_username, thread->m_username_length, thread->m_hostname,
-      thread->m_hostname_length);
+  thread->m_account = find_or_create_account(thread, &thread->m_user_name,
+                                             &thread->m_host_name);
 
-  if ((thread->m_account == nullptr) && (thread->m_username_length > 0))
-    thread->m_user = find_or_create_user(thread, thread->m_username,
-                                         thread->m_username_length);
+  if ((thread->m_account == nullptr) && (thread->m_user_name.length() > 0))
+    thread->m_user = find_or_create_user(thread, &thread->m_user_name);
 
-  if ((thread->m_account == nullptr) && (thread->m_hostname_length > 0))
-    thread->m_host = find_or_create_host(thread, thread->m_hostname,
-                                         thread->m_hostname_length);
+  if ((thread->m_account == nullptr) && (thread->m_host_name.length() > 0))
+    thread->m_host = find_or_create_host(thread, &thread->m_host_name);
 }
 
 static void fct_update_mutex_derived_flags(PFS_mutex *pfs) {
